# -*- coding: utf-8 -*-
from setuphelpers import *
import json

uninstallkey = []

# Declaring specific app values (TO CHANGE)
bin_name_string = 'VSCodeSetup-ia32-%s.exe'
# Default arguments : addcontextmenufiles,addcontextmenufolders,addtopath,associatewithfiles,!desktopicon,!quicklaunchicon,!runcode
silent_args = '/verysilent /mergetasks="addcontextmenufiles,addcontextmenufolders,addtopath,associatewithfiles,!desktopicon,!quicklaunchicon,!runcode"'
app_uninstallkey = '{F8A2A208-72B3-4D61-95FC-8A65D340689B}_is1'
app_dir = makepath(programfiles,'Microsoft VS Code')
app_bin_from_path = 'code'
ext_python = "ms-python-release.vsix"
ext_path = makepath(app_dir,ext_python)
url_api = 'https://update.code.visualstudio.com/api/update/win32/stable/VERSION'
#url_dl = 'https://go.microsoft.com/fwlink/?Linkid=852157'


def install():
    # Initializing variables
    package_version = control.version.split('-',1)[0]
    bin_name = bin_name_string % package_version

    # Installing the package
    print('Installing: %s' % bin_name)
    install_exe_if_needed(bin_name,
        silentflags=silent_args,
        key=app_uninstallkey,
        min_version=package_version)

    # Copying extension files to vscodium dir
    if isfile(ext_path):
        remove_file(ext_path)
    filecopyto(ext_python,app_dir)


def session_setup():
    print('Disabling : Telemetry, check for update, updating, update note tab, etc. And minimal configuration')

    # Configuration procedure: https://developer.hyvor.com/vscode-editing-settings-json
    #C:\Users\username\AppData\Roaming\Code\User\settings.json

    # Initializing variables
    package_version = control.version.split('-',1)[0]
    user_conf_dir = makepath(user_appdata,'Code','User')
    user_conf_file = makepath(user_conf_dir,'settings.json')
    user_conf_content = """{
    "editor.minimap.enabled": false,
    "editor.wordWrap": "on",
    "extensions.autoCheckUpdates": false,
    "extensions.autoUpdate": false,
    "python.showStartPage": false,
    "remote.SSH.allowLocalServerDownload": false,
    "telemetry.enableCrashReporter": false,
    "telemetry.enableTelemetry": false,
    "update.enableWindowsBackgroundUpdates": false,
    "update.mode": "none",
    "update.showReleaseNotes": false,
    "workbench.startupEditor": "none"
}"""

    user_conf_data = json.loads(user_conf_content)
    if not isdir(user_conf_dir):
        mkdirs(user_conf_dir)
    if not isfile(user_conf_file):
        json_write_file(user_conf_file, user_conf_data)
    else:
        new_user_conf_data = json_load_file(user_conf_file)
        new_user_conf_data.update(user_conf_data)
        json_write_file(user_conf_file, new_user_conf_data)

    # Installing extensions and avoiding re-installs
    reg_ext_installed_for_version = registry_readstring(HKEY_CURRENT_USER, r'SOFTWARE\WAPT\Code', 'Extentions Installed For Version')
    if reg_ext_installed_for_version == '' or reg_ext_installed_for_version < package_version:
        print('Installing: Extensions')
        run_notfatal('%s --install-extension "%s"' % (app_bin_from_path,ext_path))
        registry_set(HKEY_CURRENT_USER, r'SOFTWARE\WAPT\Code', 'Extentions Installed For Version', package_version)
    else:
        print('Extensions are already installed')


def update_package():
    print('Downloading/Updating package content from upstream binary sources')

    # Initializing variables
    proxies = get_proxies()
    app_name = control.name

    # Getting latest informations from VSCode API
    print('API used is: ' + url_api)
    json_load = json.loads(wgets(url_api,proxies=proxies))
    version = json_load['name']
    url_dl = json_load['url']
    latest_bin = bin_name_string % version

    print("Latest %s version is: %s" % (app_name, version))
    print("Download url is: %s" % url_dl)

    # Downloading latest binaries
    if not isfile(latest_bin):
        print('Downloading: %s' % latest_bin)
        wget(url_dl,latest_bin, proxies=proxies)

        # Checking version from file
        version_from_file = get_version_from_binary(latest_bin)
        if version != version_from_file:
            os.rename(latest_bin,bin_name_string % version_from_file)
            version = version_from_file

        # Changing version of the package
        control.version = '%s-%s'%(version,int(control.version.split('-')[-1])+1)
        control.save_control_to_wapt()
        print('Changing version to: %s in WAPT\\control' % control.version)

    # Deleting outdated binaries
    remove_outdated_binaries(version)

    # Downloading latest extensions
    json_load = json.loads(wgets('https://api.github.com/repos/microsoft/vscode-python/releases/latest', proxies=proxies))
    for download_ext in json_load['assets']:
        if download_ext['browser_download_url'].endswith('.vsix'):
            url_dl_ext = download_ext['browser_download_url']
            if isfile(ext_python):
                remove_file(ext_python)
            print('Downloading: ' + ext_python)
            wget(url_dl_ext,ext_python, proxies=proxies)
            break




def get_proxies():
    import platform
    if platform.python_version_tuple()[0] == '3':
        from urllib.request import getproxies
    else:
        from urllib import getproxies
    return getproxies()


def get_version_from_binary(filename):
    if filename.endswith('.msi'):
        return get_msi_properties(filename)['ProductVersion']
    else:
        return get_file_properties(filename)['ProductVersion']


def remove_outdated_binaries(version,list_filename_contain=('x64','x86','win32','win64','32','64'),list_extensions=('exe','msi','deb','rpm','dmg','pkg')):
    for file_ext in list_extensions:
        for bin_in_dir in glob.glob('*.%s' % file_ext):
            if not version in bin_in_dir:
                for filename_contain in list_filename_contain:
                    if not filename_contain in bin_in_dir:
                        print('Outdated binary: %s Removed' % bin_in_dir)
                        remove_file(bin_in_dir)


def json_load_file(json_file):
    with open(json_file) as read_file:
        data = json.load(read_file)
    return data


def json_write_file(json_file,data,sort_keys=True, indent=4):
    import codecs
    with codecs.open(json_file, 'w', encoding='utf-8') as write_file:
        json.dump(data, write_file, sort_keys=sort_keys, indent=indent)

